#!/usr/bin/python
# To count stats for data and store it

import json
import os
import shutil
import sys
import tempfile

def main():

    jsons = [x for x in os.listdir('/data/vision/labeling/done/') if x.endswith('json')]

    output_dictionary = dict()

    print("Starting scanning.")
    for f in jsons:

        with open('{}/{}'.format('/data/vision/labeling/done', f), 'r') as f:
            images = json.load(f)

        for image in images:
            if 'annotations' not in image:
                continue

            # Count the annotations (total and correct) marked for each image.
            for annotation in image['annotations']:
                label_type = annotation['class']
                if label_type not in output_dictionary:
                    output_dictionary[label_type] = {'count': 0,
                                                     'good': 0}

                output_dictionary[label_type]['count'] += 1
                if 'status' in image and image['status'] == 'Good':
                    output_dictionary[label_type]['good'] += 1

    with open("/data/vision/labeling/done/count/count.json", 'w') as m:
        json.dump(output_dictionary, m, indent=4)
    print("\nDone.")

if __name__ == '__main__':
    main()



#!/usr/bin/python
# This script is designed to remove bad images given tar name and json file to be present

import argparse
import json
import os
import sys
import shutil
import glob
import tarfile

# Main function
def app(args):
    if args.remote:
        path = "/data/vision/labeling/in_progress/clarification/temp/"
    else:
        path = ""
    base_name = os.path.splitext(os.path.basename(args.tarname))[0]
    print("Working on {}".format(args.tarname))
    os.rename(path + base_name + ".json",path + base_name + "-delete.json")

    # Extract tar
    print("Extracting")
    with tarfile.TarFile(path + args.tarname) as tf:
        tf.extractall(path)
    os.remove(path + args.tarname)
    try:
        os.remove("{}/{}".format(path + base_name, base_name + ".json"))
    except:
        pass
    with open(path + base_name + '-delete.json', 'r') as f:
        json_contents = json.load(f)

    # Looping over to see if there are bad images, and see if the tar is complete
    for annotation in json_contents:
        try:
            ann = annotation['status']
            if ann == 'Bad':
                print("Removing {}".format(annotation['filename']))
                os.remove("{}/{}".format(path + base_name, annotation['filename']))
        except:
            pass
    annotations = []
    print("Creating new json")
    for f in sorted(glob.glob('{}/*.jpg'.format(path + base_name))):
        annotations.append({'annotations': [],
                            'class': 'image',
                            'filename': os.path.basename(f),
                            'unlabeled': True})
    os.remove(path + base_name + '-delete.json')
    with open("{}/{}".format(path + base_name, base_name + '.json'), 'w') as f:
        json.dump(annotations, f, indent=4)
    print("Putting everything back into tar")
    with tarfile.open(path + base_name + '.tar', "w") as tar:
        tar.add(path + base_name, arcname=base_name)
    shutil.rmtree(path + base_name)
    print("Putting tar up for labeling")
    os.rename(path + base_name + '.tar', "/data/vision/labeling/new/{}".format(base_name + '.tar'))
    print("Done!")
if __name__ == '__main__':
    # Arguement handler
    parser = argparse.ArgumentParser(description='Utility for deleting bad images.')
    parser.add_argument('tarname', type=str, help='Name of the tar with bad images')
    parser.add_argument('--remote', action='store_true', help='To run remotely by other script')
    parser.set_defaults(func=app)
    args = parser.parse_args()
    args.func(args)



#!/usr/bin/python
"""
@author Ryan Summers
@date 2-27-2018

@brief Gets information about available datasets on the robosub server.
"""

from __future__ import print_function

import json
import os
import progressbar
import pysftp
import shutil
import sys
import tempfile

try:
    input = raw_input
except:
    pass

# download progress bar
bar = progressbar.ProgressBar()

# update progress bar
def progress(done, total):
    bar.update(done)

def get_json_stats(sftp):
    """ Collects metadata about the completed datasets. """

    print('Grabbing JSON information...')
    global bar
    total_size = sftp.stat("count/count.json")
    bar = progressbar.ProgressBar(max_value=total_size.st_size)
    sftp.get("count/count.json", callback=progress)
    bar.finish()

    output_dictionary = dict()

    with open("count.json", 'r') as f:
        output_dictionary = json.load(f)

    os.remove("count.json")

    return output_dictionary


def app(args):
    """ Main entry point for returning data to the server.

    Arguments
        args: [Unused]

    """
    password = os.environ.get('ROBOSUB_SFTP_PASSWORD')
    if password is None:
        print('To suppress this prompt, please set the ROBOSUB_SFTP_PASSWORD ',
                end='')
        print('environment variable.')
        password = input('Please enter the Robosub SFTP password: ')

    # Open an SFTP connection with the robosub server.
    with pysftp.Connection('robosub.eecs.wsu.edu',
                username='sftp_user',
                password=password,
                default_path='/data/vision/labeling') as sftp:

        # Check to see if there is an in-progress labeling or validation
        # session.
        with sftp.cd('in_progress/labeling/'):
            current_labeling_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        with sftp.cd('in_progress/validation/'):
            current_validation_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        with sftp.cd('in_progress/clarification/'):
            current_clarification_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        with sftp.cd('new/'):
            new_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        with sftp.cd('unvalidated/'):
            unvalidated_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        with sftp.cd('done/'):
            done_tars = [x for x in sftp.listdir() if x.endswith('.tar')]
            done_dict = get_json_stats(sftp)

        with sftp.cd('clarification/'):
            clarified_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        print('Image sets waiting to be labeled: {}'.format(len(new_tars)))
        print('Image sets waiting to be validated: {}'.format(len(unvalidated_tars)))
        print('Image sets waiting to be clarified: {}'.format(len(clarified_tars)))
        print('Image sets being validated: {}'.format(len(current_validation_tars)))
        print('Image sets being labeled: {}'.format(len(current_labeling_tars)))
        print('Image sets being clarified: {}'.format(len(current_clarification_tars)))
        print('')
        print('Done dataset stats:')
        print('Label\tNumber\tCorrect')
        for key in done_dict:
            print('{}\t{}\t{}'.format(key, done_dict[key]['count'], done_dict[key]['good']))



#!/usr/bin/python
"""
@author Ryan Summers
@date 3-14-2018

@brief Provides a UI for marking labels as valid or invalid.
"""

from __future__ import print_function

enabled = True

try:
    import cv2
except:
    print('Failed to import OpenCV2. Marker command disabled.')
    print('Please install OpenCV2 manually to enable data validation.')
    print(' - pip install opencv-python')
    enabled = False

import json
import logging
import operator
import os
import shutil
import sys
import yaml


annotation_config = dict()
"""
A dictionary of colors and alpha values for specific label names.
"""


default_colors = [(255, 0, 0),
                  (0, 255, 0),
                  (0, 0, 255),
                  (0, 255, 255),
                  (255, 0, 255),
                  (0x66, 0, 0x66),
                  (0, 0x33, 0),
                  (0, 0, 0x99),
                  (0, 0, 0),
                  (255, 255, 255),
                  (0xcc, 0, 0x66),
                  (0xff, 0xcc, 0),
                  (0x99, 0, 0x99)]
"""
A list of default colors to use in case they are not specified in the config.
"""


default_color_index = 0
"""
The default color index for automatically assigning label colors.
"""


def show_help():
    """ Displays the keyboard controls for image control. """
    print('Controls:')
    print('\ta         - Go back to the previous image.')
    print('\te/q       - Exit the application.')
    print('\ts         - Skip current image (Does not mark)')
    print('\tb         - Marks an image as \'Bad\'')
    print('\tg/[space] - Marks an image as \'Good\'')


def annotation_class_to_color(annotation_name):
    """
    Checks for alpha and overlay color for a specific annotation. If one was
        not specified in a configuration file, one is automatically assigned.

    Return:
        The color as a tuple and the alpha as a normalized value.
    """
    global default_color_index
    global annotation_config

    try:
        color = annotation_config[annotation_name]['color']
    except KeyError:
        color = default_colors[default_color_index]
        default_color_index += 1
        if default_color_index >= len(default_colors):
            default_color_index = 0

        annotation_config[annotation_name] = dict()
        annotation_config[annotation_name]['color'] = color

    try:
        alpha = annotation_config[annotation_name]['alpha']
    except KeyError:
        annotation_config[annotation_name]['alpha'] = 0.35
        alpha = annotation_config[annotation_name]['alpha']

    return color, alpha


class PhotoShow:
    """ Handles annotating and displaying an image.

    Attributes:
        img: The original OpenCV image. This may be modified by functions.
        annotations: A list of annotations for the image from Sloth.
        scale: The scale of the image that is displayed to the user.
    """

    def __init__(self, img_name, annotations, scale=0.6):
        """ Initializes the image display.

        Args:
            img_name The filename fo the image to use.
            annotations: A list of the annotations for the image from Sloth.
            scale: The scale of the image display from original size.
        """
        self.img = cv2.imread(img_name)
        self.annotations = annotations
        self.scale = scale


    def draw_annotations(self):
        """ Draws the annotations onto the source image. """
        for annotation in self.annotations:
            color, alpha = annotation_class_to_color(annotation['class'])

            origin = (int(annotation['x']), int(annotation['y']))
            end = tuple(map(operator.add, origin,
                        (int(annotation['width']), int(annotation['height']))))

            # Draw an overlay box and add it to the annotated image.
            overlay = self.img.copy()
            cv2.rectangle(overlay, origin, end, color, -1)
            self.img = cv2.addWeighted(self.img, 1 - alpha, overlay, alpha, 0)
            cv2.putText(self.img, annotation['class'], origin, \
                        cv2.FONT_HERSHEY_SIMPLEX, 1, color, 2)
        return


    def display(self):
        """ Displays a scaled version of the source image.

        Returns:
            An ASCII code of the key pressed on the OpenCV window.
        """
        img_small = cv2.resize(self.img, (0,0), fx=self.scale, fy=self.scale)

        cv2.namedWindow('Image Annotated')
        cv2.imshow('Image Annotated', img_small)
        code = cv2.waitKey()

        return chr(code & 0xFF)


def app(args):
    """ Main application entry point.

    Args:
        args: Provided by argparse command line arguments.
    """
    if not enabled:
        print('marker command disabled. Please install opencv-python')
        print(' - pip install opencv-python')
        return

    # Load an optionally provided configuration file.
    global annotation_config
    if args.config is not None:
        with open(args.config, 'r') as f:
            annotation_config = yaml.load(f)
        print(annotation_config)

    # Load the annotations from the provided file.
    with open(args.annotations, 'r') as annotation_file:
        annotations = json.load(annotation_file)

    # Backup the annotations to an original annotation file.
    fname = os.path.splitext(args.annotations)[0]
    backup_fname = '{}-original.json'.format(fname)
    shutil.copyfile(args.annotations, backup_fname)

    # Determine the proper starting index.
    i = 0
    if args.beginning is False:
        for j in range(0, len(annotations)):
            try:
                if os.path.isfile(annotations[j]['filename']):
                    status = annotations[j]['status']
            except KeyError:
                i = j
                break

    annotation_path = os.path.dirname(args.annotations)
    if annotation_path == '':
        annotation_path = '.'

    # Loop through every image specified in the annotation file.
    count = 0
    history = []
    show_help()
    while i < len(annotations):
        annotation = annotations[i]

        if annotation['class'] != 'image':
            logging.warning('Non image annotation found: {}'.format(annotation))
            i = i + 1
            continue

        # If the label filter is active, ensure that the current image contains
        # the label. If it does not contain the label, skip it.
        if args.label is not None:
            show = False
            for label in annotation['annotations']:
                if label['class'] == args.label:
                    show = True
                    break

            if show is False:
                i = i + 1
                continue

        # Get input from the user about the status of the image if it is being
        # displayed.
        try:
            current_img = PhotoShow(
                    '{}/{}'.format(annotation_path, annotation['filename']),
                    annotation['annotations'],
                    scale=args.scale)
        except:
            logging.warning('Failed to read {}'.format(annotation['filename']))
            i = i + 1
            continue

        # Draw the annotations onto the current image and display it.
        current_img.draw_annotations()
        status = current_img.display()

        # Figure out what the user wants to do with the image and respond
        # accordingly.
        if status == 'b':
            print('BAD: {}'.format(annotation['filename']))
            annotations[i]['status'] = 'Bad'

        elif status == 'g' or status == ' ':
            annotations[i]['status'] = 'Good'

        elif status == 'a':
            if len(history) > 0:
                i = history[-1]
                history = history[:-1]
                continue
            else:
                print('There is no previous image to display.')
            break

        elif status == 's':
            print('SKIP: {}'.format(annotation['filename']))
            break

        elif status == 'e' or status == 'q':
            break

        else:
            show_help()
            continue

        # Increment the image index.
        history.append(i)
        i = i + 1

    print('{} / {} labels marked.'.format(i, len(annotations)))

    # Save the updated JSON to the annotation file.
    with open(args.annotations, 'w') as output:
        json.dump(annotations, output, indent=2)

    print('Original JSON is available at {}'.format(backup_fname))
    print('Validation JSON saved to {}'.format(args.annotations))



#!/usr/bin/python
"""
@author Ryan Summers
@date 2-27-2018

@brief Collects all completed tarballs into a single labelled dataset.
"""

from __future__ import print_function

import glob
import json
import os
import progressbar
import pysftp
import shutil
import tarfile
import tempfile

try:
    input = raw_input
except:
    pass


def app(args):
    """ The main entry point to the collection application.

    Arguments
        arg: A structure passed in from argparse containing a single named
             element, tarball. Tarball indicates the location to save the final
             unified tarball locally.
    """

    password = os.environ.get('ROBOSUB_SFTP_PASSWORD')
    if password is None:
        print('To suppress this prompt, please set the ROBOSUB_SFTP_PASSWORD ',
                end='')
        print('environment variable.')
        password = input('Please enter the Robosub SFTP password: ')

    # Generate the SFTP connection to the robosub server for grabbing tarballs.
    with pysftp.Connection('robosub.eecs.wsu.edu',
                username='sftp_user',
                password=password,
                default_path='/data/vision/labeling/done') as sftp:
        files = sftp.listdir()

        # Determine which tarballs are valid labelled datasets by intersecting
        # available JSONs and tar archives.
        tarballs = [x for x in files if x.endswith('.tar')]
        jsons = [x for x in files if x.endswith('.json')]

        tar_bases = [os.path.splitext(x)[0] for x in tarballs]
        json_bases = [os.path.splitext(x)[0] for x in jsons]

        groups = set(tar_bases).intersection(json_bases)

        # Download the tarballs into a temporary directory.
        tar_directory = tempfile.mkdtemp()

        print('Downloading tarballs...')
        download_bar = progressbar.ProgressBar(max_value=len(groups))
        for i, tar_base in enumerate(groups):
            download_bar.update(i)
            tar_name = tar_base + '.tar'
            json_name = tar_base + '.json'
            sftp.get(tar_name, localpath='{}/{}'.format(tar_directory,
                                                        tar_name))
            sftp.get(json_name, localpath='{}/{}'.format(tar_directory,
                                                         json_name))

        download_bar.finish()

    # Untar everything into a temporary working directory
    working_directory = tempfile.mkdtemp()

    print('Combining tarballs...')
    bar = progressbar.ProgressBar(max_value=len(groups))
    annotations = []
    for i, dataset in enumerate(groups):
        bar.update(i)
        json_fname = tar_directory + '/' + dataset + '.json'
        tar_fname = tar_directory + '/' + dataset + '.tar'

        # Extract the tarball and read the annotations.
        with tarfile.TarFile(tar_fname, mode='r') as tf:
            tf.extractall(working_directory)

        with open(json_fname, 'r') as f:
            new_annotations = json.load(f)

        tar_dir = os.path.splitext(os.path.basename(tar_fname))[0]

        # Prepend the tar directory name with each annotation filename to keep
        # the original directory structure.
        for annotation in new_annotations:
            fname = annotation['filename']
            annotation['filename'] = tar_dir + '/' + fname

        annotations += new_annotations

    bar.finish()

    # Write all of the annotations with the new directory structure into the
    # final tarball.
    with open('{}/labels.json'.format(working_directory), 'w') as f:
        json.dump(annotations, f, indent=4)

    print('Writing final tarball...')
    with tarfile.TarFile(args.tarball, mode='w') as tf:
        tf.add(working_directory, arcname='dataset')

    # Remove the temporary working directories.
    shutil.rmtree(working_directory)
    shutil.rmtree(tar_directory)

    print('Tarfile saved to {}.'.format(args.tarball))



#!/usr/bin/python
"""
@author Ryan Summers
@date 4-4-2018

@brief Gets statistics about labeling progress for all users.
"""

from __future__ import print_function

import glob
import json
import os
import pysftp
import shutil
import tempfile

try:
    input = raw_input
except:
    pass


def app(args):
    """ Main entry point for returning data to the server.

    Arguments
        args: [Unused]

    """
    password = os.environ.get('ROBOSUB_SFTP_PASSWORD')
    if password is None:
        print('To suppress this prompt, please set the ROBOSUB_SFTP_PASSWORD ',
                end='')
        print('environment variable.')
        password = input('Please enter the Robosub SFTP password: ')

    # Open an SFTP connection with the robosub server.
    with pysftp.Connection('robosub.eecs.wsu.edu',
                username='sftp_user',
                password=password,
                default_path='/data/vision/labeling') as sftp:

        label_stats = dict()
        dname = tempfile.mkdtemp()
        sftp.get_d('history', dname)
        for fname in glob.glob('{}/*.log'.format(dname)):
            with open(fname, 'r') as f:
                data = json.load(f)
            for entry in data:
                user = entry['owner']
                log = entry['stats']
                if user not in label_stats:
                    label_stats[user] = log
                else:
                    for key in log:
                        label_stats[user][key] += log[key]
        shutil.rmtree(dname)

    headings = ['User', 'Images Labeled', 'Labels Added', 'Images Validated', 'Labels Validated']
    row_format = '{:>20}' * len(headings)
    print(row_format.format(*headings))

    for user in label_stats:
        images_labeled = label_stats[user]['images_labeled']
        labels_added = label_stats[user]['labels_added']
        images_validated = label_stats[user]['images_validated']
        labels_validated = label_stats[user]['labels_validated']
        print(row_format.format(user, images_labeled, labels_added, images_validated, labels_validated))



#!/usr/bin/python
"""
@author Ryan Summers
@date 2-27-2018

@brief Returns labeled or validated datasets to the robosub server.
"""

from __future__ import print_function

import datetime
import json
import os
import pysftp
import shutil
import sys
import tempfile
import glob
import tarfile
import progressbar

try:
    input = raw_input
except:
    pass

# Upload progress bar
bar = progressbar.ProgressBar()

# update progress bar
def progress(done, total):
    bar.update(done)

def app(args):
    """ Main entry point for returning data to the server.

    Arguments:
        args: Passed in from argparse. Must have a named member `annotations`
              that specifies the path to the annotation JSON.
    """

    # Grab the username for data ownership and the SFTP password.
    data_owner = os.environ.get('ROBOSUB_WHO_AM_I')
    if data_owner is None:
        print('To suppress this prompt, please set the environment variable ',
                end='')
        print('ROBOSUB_WHO_AM_I to your name.')
        data_owner = input('Please enter your name (First Last): ')

    password = os.environ.get('ROBOSUB_SFTP_PASSWORD')
    if password is None:
        print('To suppress this prompt, please set the ROBOSUB_SFTP_PASSWORD ',
                end='')
        print('environment variable.')
        password = input('Please enter the Robosub SFTP password: ')

    # Read the annotations to check if validation or labeling have been
    # completed.
    with open(args.annotations, 'r') as f:
        json_contents = json.load(f)
    tar_name = os.path.splitext(os.path.basename(args.annotations))[0] + '.tar'

    # Open an SFTP connection with the robosub server.
    with pysftp.Connection('robosub.eecs.wsu.edu',
                username='sftp_user',
                password=password,
                default_path='/data/vision/labeling') as sftp:

        # Check to see if there is an in-progress labeling or validation
        # session.
        with sftp.cd('in_progress/labeling/'):
            labeling_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        with sftp.cd('in_progress/validation/'):
            validation_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        with sftp.cd('in_progress/clarification/'):
            clarification_tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        in_validation = False
        in_clarification = False
        delete = False
        bad_count = 0

        # If the dataset is currently being labeled, set up the proper source
        # and destination paths on the server. If labeling, validation or
        # clarification is not fully completed, move from in_progress back
        # into the intermediate step.
        if tar_name in labeling_tars:

            complete = True
            for annotation in json_contents:
                try:
                    complete = not annotation['unlabeled']
                    if not complete:
                        break
                except:
                    pass

            src_dir = 'in_progress/labeling/'
            dest_dir = 'unvalidated/' if complete else 'new/'
        elif tar_name in validation_tars:

            in_validation = True
            complete = True
            for annotation in json_contents:
                try:
                    status = annotation['status']
                except:
                    complete = False
                    break
            src_dir = 'in_progress/validation/'
            dest_dir = 'done/' if complete else 'unvalidated/'
        elif tar_name in clarification_tars:

            in_clarification = True
            complete = True
            dir_name = os.path.dirname(args.annotations)

            # Looping over to see if there are bad images, and see if the tar is complete
            for annotation in json_contents:
                try:
                    ann = annotation['status']
                    if ann == 'Bad':
                        delete = True
                        bad_count += 1
                except:
                    complete = False
                    pass

            if not delete:
                src_dir = 'in_progress/clarification/'
                dest_dir = 'new/' if complete else 'clarification/'
            else:
                src_dir = 'in_progress/clarification/'
                dest_dir = 'in_progress/clarification/temp/' if complete else 'clarification/'
        else:
            print('The supplied JSON name does not match any in-progress ',
                    end='')
            print('validation, labeling sessions or clarification.')
            print('Current labeling sessions: {}'.format(labeling_tars))
            print('Current validation sessions: {}'.format(validation_tars))
            print('Current clarification sessions: {}'.format(clarification_tars))
            sys.exit(-1)

        if not complete:
            if in_validation:
                print('Validation was not completed. Returning progress to ',
                        end='')
                print('unvalidated datasets.')
            elif in_clarification:
                print('clarification was not completed. Returning progress to new ',
                        end='')
                print('datasets.')
            else:
                print('Labeling was not completed. Returning progress to new ',
                        end='')
                print('datasets.')

        with sftp.cd(src_dir):
            tars = [x for x in sftp.listdir() if x.endswith('.tar')]

        if tar_name not in tars:
            print('The provided annotations dataset was not found in ',
                    end='')
            print('robosub.eecs.wsu.edu:/data/vision/labeling/' + src_dir)
            sys.exit(-1)

            # If the annotations already exist on the server, pull them down to
            # figure out what new labels were added for stat tracking.
        if tar_name not in clarification_tars:
            previous_annotations = []
            with sftp.cd(src_dir):
                if os.path.basename(args.annotations) in sftp.listdir():
                    with tempfile.NamedTemporaryFile() as tempf:
                        sftp.get(os.path.basename(args.annotations), tempf.name)
                        with open(tempf.name, 'r') as f:
                            previous_annotations = json.load(f)

            with open(args.annotations, 'r') as f:
                new_annotations = json.load(f)

            if len(previous_annotations) and len(new_annotations) != len(previous_annotations):
                print('Provided annotation file and server annotation file differ.')
                sys.exit(-1)

            log = {'labels_added': 0,
                   'images_labeled': 0,
                   'labels_validated': 0,
                   'images_validated': 0}

            if len(previous_annotations) == 0:
                for annotation in new_annotations:
                    if in_validation:
                        log['images_validated'] += 1
                        log['labels_validated'] += len(annotation['annotations'])
                    else:
                        labels_added = len(annotation['annotations'])

                        log['labels_added'] += labels_added
                        if labels_added > 0:
                            log['images_labeled'] += 1
            else:
                for old, new in zip(previous_annotations, new_annotations):
                    if in_validation:
                        log['images_validated'] += 1
                        try:
                            status = new['status']
                            try:
                                old_status = old['status']
                                if old_status != status:
                                    log['labels_validated'] += len(new['annotations'])
                                    log['images_validated'] += 1
                            except:
                                log['labels_validated'] += len(new['annotations'])
                                log['images_validated'] += 1
                        except:
                            pass
                    else:
                        labels_added = len(new['annotations']) - len(old['annotations'])
                        if labels_added > 0:
                            log['labels_added'] += labels_added
                            log['images_labeled'] += 1

        # Upload the JSON to the server. Or tar if the images were bad
        # in clarification proccess
        if delete or (tar_name not in clarification_tars):
            with sftp.cd(dest_dir):

                # Return tar file
                global bar
                total_size = os.stat(args.annotations)
                bar = progressbar.ProgressBar(max_value=total_size.st_size)
                sftp.put(args.annotations, callback=progress)
                bar.finish()

        # Move the tar from in_progress to the proper destination.
        # or delete tar if images were bad in clarification
        sftp.rename('{}/{}'.format(src_dir, tar_name),
                    '{}/{}'.format(dest_dir, tar_name))

        if delete and complete:
            print('Found {} bad images. Deleting'.format(bad_count))
            with sftp.cd(dest_dir):
                sftp.execute("python /data/vision/labeling/in_progress/clarification/temp/delete.py {} --remote".format(tar_name))
                print('Deleted, taring up remaining images and puting into labeling folder.')
        # Remove the ownership and annotation files.
        with sftp.cd(src_dir):
            if os.path.basename(args.annotations) in sftp.listdir():
                sftp.remove(os.path.basename(args.annotations))
            sftp.remove('{}.owner'.format(tar_name))

        # Delete the folder containing the JSON if the user would like. Ensure
        # the dirname is a valid path.
        directory = os.path.dirname(args.annotations)
        if directory != '':
            if args.auto_delete:
                delete = True
            else:
                user_input = input('Delete folder `{}/`? (y/n): '.format(
                            directory))
                delete = user_input == 'y' or user_input == 'Y'

            if delete:
                print('Deleting {}/'.format(directory))
                shutil.rmtree(directory)

            # Finally, upload the stats to the history folder on the
            # server for stats tracking.
        if tar_name not in clarification_tars:
            stats = [{'owner': data_owner,
                      'stats': log,
                      'date-time': datetime.datetime.now().isoformat()}]

            # Only upload stats if someone has modified the annotations.
            if log['images_labeled'] != 0 or log['images_validated'] != 0:
                with tempfile.NamedTemporaryFile(prefix=data_owner) as tempf:
                    with open(tempf.name, 'w') as f:
                        json.dump(stats, f)

                    with sftp.cd('history'):
                        log_files = [x for x in sftp.listdir() if x.startswith(data_owner.replace(' ', '_'))]
                        file_numbers = [int(os.path.splitext(x)[0].split('-')[-1]) for x in log_files]
                        if len(file_numbers) == 0:
                            f_number = 0
                        else:
                            f_number = max(file_numbers) + 1

                        dst = os.path.basename(tempf.name)
                        sftp.put(tempf.name)
                        sftp.rename(dst, '{}-{}.log'.format(data_owner.replace(' ', '_'), f_number))

        sftp.execute("python /data/vision/labeling/done/count/count.py") # Update Stats on the server

        print('Data has been successfully returned.')



#!/usr/bin/python

from setuptools import setup

setup(
    # Application name:
    name="rslabel",

    # Version number:
    version="0.9.1",

    # Application author details:
    author="Ryan Summers",
    author_email="summers.ryan.m@gmail.com",

    # Packages
    packages=["rslabel"],

    # Details
    url="http://robosub.eecs.wsu.edu/wiki/cs/vision/image_tagging/start#data-management",

    #
    # license="LICENSE.txt",
    description="Used for Palouse Robosub image labeling management.",

    long_description=open("README.txt").read(),

    scripts=['bin/rslabel'],

    # Dependent packages (distributions)
    install_requires=[
        "pysftp",
        "progressbar2",
    ],
)



#!/usr/bin/python
"""
@author Ryan Summers
@date 2-27-2018

@brief Unbags images from a ROS Bag and uploads them to the Robosub server.
"""

from __future__ import print_function

enabled = True

import glob
import json
import os
import progressbar
import pysftp
import shutil
import tarfile
import tempfile

try:
    import cv2
except:
    print('OpenCV installation not detected. Upload command disabled.')
    enabled = False

try:
    import rosbag
    import cv_bridge
except:
    print('ROS installation not detected. Upload command disabled')
    enabled = False

try:
    input = raw_input
except:
    pass


def unbag_images(bag, directory):
    """ Unbags images into a directory.

    Args:
        bag: The RosBag to unbag.
        directory: The path to the directory to store images into.
    """

    bridge = cv_bridge.CvBridge()
    msg_count = bag.get_message_count(topic_filters=['/camera/left/undistorted',
            '/camera/right/undistorted',
            '/camera/bottom/undistorted'])
    bar = progressbar.ProgressBar(max_value=msg_count)
    i = 0
    print('Unbagging images...')
    for topic, msg, t in bag.read_messages(
            topics=['/camera/left/undistorted',
                    '/camera/right/undistorted',
                    '/camera/bottom/undistorted']):
        bar.update(i)
        i += 1
        cv_image = bridge.imgmsg_to_cv2(msg, 'bgr8')
        prefix = 'left' if 'left' in topic else 'right' if 'right' in topic else 'bottom'
        timestr = '%.6f' % msg.header.stamp.to_sec()
        image_name = '{}/{}_{}.jpg'.format(directory, prefix, timestr)
        cv2.imwrite(image_name, cv_image)
    bar.finish()


def app(args):
    """ Main entry point to the unbag_upload application.

    Arguments
        args: Passed in by argparse. Must have member `bag_file` that denotes
              the path to the Rosbag. Must have member `files_per_tar` that
              denotes how many images should be put into each archive.
    """
    if not enabled:
        print('The upload command is not enabled because ROS or OpenCV is not')
        print('properly installed on the host system.')
        return

    password = os.environ.get('ROBOSUB_SFTP_PASSWORD')
    if password is None:
        print('To suppress this prompt, please set the ROBOSUB_SFTP_PASSWORD ',
                end='')
        print('environment variable.')
        password = input('Please enter the Robosub SFTP password: ')

    working_directory = tempfile.mkdtemp(dir='/tmp')

    # Unbag all of the images into a temporary directory.
    ros_bag = rosbag.Bag(args.bag_file, 'r')
    temp_dir = tempfile.mkdtemp(dir=working_directory)
    unbag_images(ros_bag, temp_dir)

    print('Images unbagged to {}'.format(temp_dir))

    # Get a list of all the unbagged images.
    images = glob.glob('{}/*.jpg'.format(temp_dir))
    images.sort()

    bag_base = os.path.splitext(os.path.basename(args.bag_file))[0]

    # Move the jpgs into individual archives.
    print('Creating image archives...')
    bar = progressbar.ProgressBar(max_value=len(images))

    for i in range(0, len(images), args.files_per_tar):
        bar.update(i)
        end = i +  args.files_per_tar
        if end >= len(images):
            end = len(images)

        image_dir = '{}/{}_{}'.format(working_directory, bag_base, i)
        os.mkdir(image_dir)

        annotations = []

        # Loop through all of the images in the current step. Move them into a
        # directory for archiving.
        for img in images[i : end]:
            shutil.move(img, image_dir)

        # Create an archive with the JSON annotation file and the images.
        with tarfile.TarFile('{}/{}.tar'.format(working_directory,
                                os.path.basename(image_dir)), mode='w') as tf:
            tf.add(image_dir, arcname=os.path.basename(image_dir))

        # Remove the image directory after we have archived it.
        shutil.rmtree(image_dir)

    bar.finish()

    # Upload the tar archives to the Robosub server using the PySFTP client.
    print('Uploading archives to server...')
    files = glob.glob('{}/*.tar'.format(working_directory))
    bar_tar = progressbar.ProgressBar(max_value=len(files))
    with pysftp.Connection('robosub.eecs.wsu.edu',
            username='sftp_user',
            password=password,
            default_path='/data/vision/labeling/clarification') as sftp:
        for i, f in enumerate(files):
            bar_tar.update(i)
            sftp.put(f)
    bar_tar.finish()

    # Remove the temporary working directory we created.
    shutil.rmtree(working_directory)



#!/usr/bin/python
"""
@author Ryan Summers
@date 2-27-2018

@brief Gets data for labelling or validation from the Robosub server.
"""

from __future__ import print_function

import glob
import json
import os
import pysftp
import shutil
import sys
import tarfile
import tempfile
import progressbar

try:
    input = raw_input
except:
    pass

# download progress bar
bar = progressbar.ProgressBar()

# update progress bar
def progress(done, total):
    bar.update(done)

def app(args):
    """ Main entry point for the dataset getter application.

    Arguments
        args: Passed in from argparse. Should have a named element `validation`
              that specifies whether or not the dataset to get should be for
              validation or labeling.
    """

    if args.validation:
        src_dir = 'unvalidated/'
        dest_dir = 'in_progress/validation/'
    elif args.clarification:
        src_dir = 'clarification/'
        dest_dir = 'in_progress/clarification/'
    else:
        src_dir = 'new/'
        dest_dir = 'in_progress/labeling/'

    # Grab the username for data ownership and the SFTP password.
    data_owner = os.environ.get('ROBOSUB_WHO_AM_I')
    if data_owner is None:
        print('To suppress this prompt, please set the environment variable ',
                end='')
        print('ROBOSUB_WHO_AM_I to your name.')
        data_owner = input('Please enter your name (First Last): ')

    password = os.environ.get('ROBOSUB_SFTP_PASSWORD')
    if password is None:
        print('To suppress this prompt, please set the ROBOSUB_SFTP_PASSWORD ',
                end='')
        print('environment variable.')
        password = input('Please enter the Robosub SFTP password: ')

    # Create an SFTP connection to the robosub server for getting data.
    with pysftp.Connection('robosub.eecs.wsu.edu',
            username='sftp_user',
            password=password,
            default_path='/data/vision/labeling') as sftp:
        with sftp.cd(src_dir):
            tars = [x for x in sftp.listdir() if str(x).endswith('.tar')]

        if len(tars) == 0:
            print('No data is available at ',)
            print('robosub.eecs.wsu.edu/data/vision/labeling/' + src_dir)
            sys.exit(0)

        tar = str(tars[0])
        tar_name = os.path.splitext(tar)[0]

        print('Grabbing {}'.format(tar))

        # Move tar to in_progress - This prevents other users from working on
        # the same data as us.
        sftp.rename(src_dir + tar, dest_dir + tar)

        # Make an owner file on the server to specify who currently has the
        # dataset checked out.
        owner_file = '{}.owner'.format(tar)
        temp_dir = tempfile.mkdtemp()
        with open('{}/{}'.format(temp_dir, owner_file), 'w') as f:
            f.write(data_owner + '\n')

        with sftp.cd(dest_dir):
            sftp.put('{}/{}'.format(temp_dir, owner_file))

            # Get the tar and untar it.
            global bar
            total_size = sftp.stat(tar)
            bar = progressbar.ProgressBar(max_value=total_size.st_size)
            sftp.get(tar, callback=progress)
            bar.finish()

        shutil.rmtree(temp_dir)

        with tarfile.TarFile(tar) as tf:
            tf.extractall()

        # If this is validation (or is partially completed), grab the JSON from
        # the server.
        tar_base = os.path.splitext(tar)[0]
        json_name = tar_base + '.json'

        if args.validation:
            sftp.rename(src_dir + json_name, dest_dir + json_name)
            with sftp.cd(dest_dir):
                print('Grabbing {}'.format(json_name))
                sftp.get(json_name,
                         localpath='{}/{}'.format(tar_base, json_name))
        else:
            with sftp.cd(src_dir):
                elements = sftp.listdir()

            if json_name in elements:
                print('Grabbing {}'.format(json_name))
                with sftp.cd(src_dir):
                    sftp.get(json_name,
                             localpath='{}/{}'.format(tar_base, json_name))
                sftp.rename('{}/{}'.format(src_dir, json_name),
                            '{}/{}'.format(dest_dir, json_name))
            else:
                # Generate labeling JSON for data to begin labeling if it is not
                # available on the server.
                print('Generating JSON for the dataset.')
                annotations = []
                for f in sorted(glob.glob('{}/*.jpg'.format(tar_name))):
                    annotations.append({'annotations': [],
                                        'class': 'image',
                                        'filename': os.path.basename(f),
                                        'unlabeled': True})

                with open('{}/{}'.format(tar_base, json_name), 'w') as f:
                    json.dump(annotations, f, indent=4)

    # Remove the tar file - We don't need it anymore.
    os.remove(tar)

    print('Images available in {}'.format(tar_name))
